@@ -20,7 +20,6 @@
 #include "openbox.h"
 #include "menu.h"
 #include "menuframe.h"
-#include "action.h"
 #include "screen.h"
 #include "client.h"
 #include "focus.h"
@@ -38,45 +37,37 @@ typedef struct
     guint desktop;
 } DesktopData;
 
+#define CLIENT -1
+#define ADD_DESKTOP -2
+#define REMOVE_DESKTOP -3
+
 static gboolean desk_menu_update(ObMenuFrame *frame, gpointer data)
 {
     ObMenu *menu = frame->menu;
     DesktopData *d = data;
     GList *it;
-    gint i;
     gboolean empty = TRUE;
     gboolean onlyiconic = TRUE;
 
     menu_clear_entries(menu);
 
-    for (it = focus_order, i = 0; it; it = g_list_next(it), ++i) {
+    for (it = focus_order; it; it = g_list_next(it)) {
         ObClient *c = it->data;
         if (client_normal(c) && (!c->skip_taskbar || c->iconic) &&
             (c->desktop == d->desktop || c->desktop == DESKTOP_ALL))
         {
-            GSList *acts = NULL;
-            ObAction* act;
             ObMenuEntry *e;
             const ObClientIcon *icon;
 
             empty = FALSE;
 
-            act = action_from_string("Activate",
-                                     OB_USER_ACTION_MENU_SELECTION);
-            act->data.activate.any.c = c;
-            acts = g_slist_append(acts, act);
-            act = action_from_string("Desktop",
-                                     OB_USER_ACTION_MENU_SELECTION);
-            act->data.desktop.desk = d->desktop;
-            acts = g_slist_append(acts, act);
-
             if (c->iconic) {
                 gchar *title = g_strdup_printf("(%s)", c->icon_title);
-                e = menu_add_normal(menu, i, title, acts, FALSE);
+                e = menu_add_normal(menu, CLIENT, title, NULL, FALSE);
                 g_free(title);
             } else {
                 onlyiconic = FALSE;
-                e = menu_add_normal(menu, i, c->title, acts, FALSE);
+                e = menu_add_normal(menu, CLIENT, c->title, NULL, FALSE);
             }
 
             if (config_menu_client_list_icons
@@ -86,41 +77,36 @@ static gboolean desk_menu_update(ObMenuFrame *frame, gpointer data)
                 e->data.normal.icon_data = icon->data;
                 e->data.normal.icon_alpha = c->iconic ? OB_ICONIC_ALPHA : 0xff;
             }
+
+            e->data.normal.data = c;
         }
     }
 
     if (empty || onlyiconic) {
+        ObMenuEntry *e;
+
         /* no entries or only iconified windows, so add a
          * way to go to this desktop without uniconifying a window */
         if (!empty)
-            menu_add_separator(menu, -1, NULL);
+            menu_add_separator(menu, CLIENT, NULL);
 
-        GSList *acts = NULL;
-        ObAction* act;
-        ObMenuEntry *e;
-
-        act = action_from_string("Desktop", OB_USER_ACTION_MENU_SELECTION);
-        act->data.desktop.desk = d->desktop;
-        acts = g_slist_append(acts, act);
-        e = menu_add_normal(menu, 0, _("Go there..."), acts, TRUE);
+        e = menu_add_normal(menu, d->desktop, _("Go there..."), NULL, TRUE);
         if (d->desktop == screen_desktop)
             e->data.normal.enabled = FALSE;
     }
+
     return TRUE; /* always show */
 }
 
-/* executes it using the client in the actions, since we set that
-   when we make the actions! */
 static void desk_menu_execute(ObMenuEntry *self, ObMenuFrame *f,
-                              ObClient *c, guint state, gpointer data,
-                              Time time)
+                              ObClient *c, guint state, gpointer data)
 {
-    ObAction *a;
-
-    if (self->data.normal.actions) {
-        a = self->data.normal.actions->data;
-        action_run(self->data.normal.actions, a->data.any.c, state, time);
+    if (self->id == CLIENT) {
+        if (self->data.normal.data) /* it's set to NULL if its destroyed */
+            client_activate(self->data.normal.data, FALSE, TRUE, TRUE, TRUE);
     }
+    else
+        screen_set_desktop(self->id, TRUE);
 }
 
 static void desk_menu_destroy(ObMenu *menu, gpointer data)
@@ -143,7 +129,7 @@ static gboolean self_update(ObMenuFrame *frame, gpointer data)
         menu_free(desktop_menus->data);
         desktop_menus = g_slist_delete_link(desktop_menus, desktop_menus);
     }
-    
+
     for (i = 0; i < screen_num_desktops; ++i) {
         ObMenu *submenu;
         gchar *name = g_strdup_printf("%s-%u", MENU_NAME, i);
@@ -162,9 +148,27 @@ static gboolean self_update(ObMenuFrame *frame, gpointer data)
         desktop_menus = g_slist_append(desktop_menus, submenu);
     }
 
+    menu_add_separator(menu, CLIENT, NULL);
+    menu_add_normal(menu, ADD_DESKTOP, _("_Add new desktop"), NULL, TRUE);
+    menu_add_normal(menu, REMOVE_DESKTOP, _("_Remove last desktop"),
+                    NULL, TRUE);
+
     return TRUE; /* always show */
 }
 
+static void self_execute(ObMenuEntry *self, ObMenuFrame *f,
+                         ObClient *c, guint state, gpointer data)
+{
+    if (self->id == ADD_DESKTOP) {
+        screen_add_desktop(FALSE);
+        menu_frame_hide_all();
+    }
+    else if (self->id == REMOVE_DESKTOP) {
+        screen_remove_desktop(FALSE);
+        menu_frame_hide_all();
+    }
+}
+
 static void client_dest(ObClient *client, gpointer data)
 {
     /* This concise function removes all references to a closed
@@ -176,11 +180,10 @@ static void client_dest(ObClient *client, gpointer data)
         GList *eit;
         for (eit = mit->entries; eit; eit = g_list_next(eit)) {
             ObMenuEntry *meit = eit->data;
-            if (meit->type == OB_MENU_ENTRY_TYPE_NORMAL) {
-                ObAction *a = meit->data.normal.actions->data;
-                ObClient *c = a->data.any.c;
-                if (c == client)
-                    a->data.any.c = NULL;
+            if (meit->type == OB_MENU_ENTRY_TYPE_NORMAL &&
+                meit->data.normal.data == client)
+            {
+                    meit->data.normal.data = NULL;
             }
         }
     }
@@ -195,6 +198,7 @@ void client_list_menu_startup(gboolean reconfig)
 
     menu = menu_new(MENU_NAME, _("Desktops"), TRUE, NULL);
     menu_set_update_func(menu, self_update);
+    menu_set_execute_func(menu, self_execute);
 }
 
 void client_list_menu_shutdown(gboolean reconfig)
