@@ -76,6 +76,11 @@ extern gboolean config_focus_last;
 extern gboolean config_focus_under_mouse;
 
 extern ObPlacePolicy config_place_policy;
+/*! Place windows in the center of the free area */
+extern gboolean config_place_center;
+
+/*! User-specified margins around the edge of the screen(s) */
+extern StrutPartial config_margins;
 
 /*! When true windows' contents are refreshed while they are resized; otherwise
   they are not updated until the resize is complete */
@@ -151,6 +156,9 @@ extern gint config_mouse_threshold;
 /*! Number of milliseconds within which 2 clicks must occur to be a
   double-click */
 extern gint config_mouse_dclicktime;
+/*! Number of milliseconds that the mouse has to be on the screen edge before
+  a screen edge event is triggered */
+extern gint config_mouse_screenedgetime;
 
 /*! Number of pixels to resist while crossing another window's edge */
 extern gint config_resist_win;
