@@ -22,6 +22,7 @@
 #include "window.h"
 #include "openbox.h"
 #include "dock.h"
+#include "actions.h"
 #include "client.h"
 #include "xerror.h"
 #include "prop.h"
@@ -36,7 +37,6 @@
 #include "propwin.h"
 #include "mouse.h"
 #include "mainloop.h"
-#include "framerender.h"
 #include "focus.h"
 #include "focus_cycle.h"
 #include "moveresize.h"
@@ -257,12 +257,6 @@ static void event_hack_mods(XEvent *e)
         break;
     case KeyRelease:
         e->xkey.state = modkeys_only_modifier_masks(e->xkey.state);
-#ifdef XKB
-        if (XkbGetState(ob_display, XkbUseCoreKbd, &xkb_state) == Success) {
-            e->xkey.state = xkb_state.compat_state;
-            break;
-        }
-#endif
         /* remove from the state the mask of the modifier key being released,
            if it is a modifier key being released that is */
         e->xkey.state &= ~modkeys_keycode_to_mask(e->xkey.keycode);
@@ -662,7 +656,7 @@ static void event_process(const XEvent *ec, gpointer data)
         xwc.border_width = e->xconfigurerequest.border_width;
         xwc.sibling = e->xconfigurerequest.above;
         xwc.stack_mode = e->xconfigurerequest.detail;
-       
+
         /* we are not to be held responsible if someone sends us an
            invalid request! */
         xerror_set_ignore(TRUE);
@@ -695,7 +689,7 @@ static void event_process(const XEvent *ec, gpointer data)
 static void event_handle_root(XEvent *e)
 {
     Atom msgtype;
-     
+
     switch(e->type) {
     case SelectionClear:
         ob_debug("Another WM has requested to replace us. Exiting.\n");
@@ -792,7 +786,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
     ObFrameContext con;
     static gint px = -1, py = -1;
     static guint pb = 0;
-     
+
     switch (e->type) {
     case ButtonPress:
         /* save where the press occured for the first button pressed */
@@ -809,8 +803,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
            want to deal with them
         */
         if (!(e->xbutton.button == 4 || e->xbutton.button == 5) &&
-            !keyboard_interactively_grabbed() &&
-            !menu_frame_visible)
+            !grab_on_keyboard())
         {
             /* use where the press occured */
             con = frame_context(client, e->xbutton.window, px, py);
@@ -823,23 +816,23 @@ static void event_handle_client(ObClient *client, XEvent *e)
             switch (con) {
             case OB_FRAME_CONTEXT_MAXIMIZE:
                 client->frame->max_press = (e->type == ButtonPress);
-                framerender_frame(client->frame);
+                frame_adjust_state(client->frame);
                 break;
             case OB_FRAME_CONTEXT_CLOSE:
                 client->frame->close_press = (e->type == ButtonPress);
-                framerender_frame(client->frame);
+                frame_adjust_state(client->frame);
                 break;
             case OB_FRAME_CONTEXT_ICONIFY:
                 client->frame->iconify_press = (e->type == ButtonPress);
-                framerender_frame(client->frame);
+                frame_adjust_state(client->frame);
                 break;
             case OB_FRAME_CONTEXT_ALLDESKTOPS:
                 client->frame->desk_press = (e->type == ButtonPress);
-                framerender_frame(client->frame);
-                break; 
+                frame_adjust_state(client->frame);
+                break;
             case OB_FRAME_CONTEXT_SHADE:
                 client->frame->shade_press = (e->type == ButtonPress);
-                framerender_frame(client->frame);
+                frame_adjust_state(client->frame);
                 break;
             default:
                 /* nothing changes with clicks for any other contexts */
@@ -957,7 +950,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
                           (e->type == EnterNotify ? "Enter" : "Leave"),
                           e->xcrossing.mode,
                           e->xcrossing.detail, (client?client->window:0));
-            if (keyboard_interactively_grabbed())
+            if (grab_on_keyboard())
                 break;
             if (config_focus_follow && config_focus_delay &&
                 /* leave inferior events can happen when the mouse goes onto
@@ -1000,7 +993,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
             frame_adjust_state(client->frame);
             break;
         case OB_FRAME_CONTEXT_FRAME:
-            if (keyboard_interactively_grabbed())
+            if (grab_on_keyboard())
                 break;
             if (e->xcrossing.mode == NotifyGrab ||
                 e->xcrossing.mode == NotifyUngrab ||
@@ -1068,26 +1061,34 @@ static void event_handle_client(ObClient *client, XEvent *e)
         if (e->xconfigurerequest.value_mask & CWStackMode) {
             ObClient *sibling = NULL;
             gulong ignore_start;
+            gboolean ok = TRUE;
 
             /* get the sibling */
             if (e->xconfigurerequest.value_mask & CWSibling) {
                 ObWindow *win;
                 win = g_hash_table_lookup(window_map,
                                           &e->xconfigurerequest.above);
-                if (WINDOW_IS_CLIENT(win) && WINDOW_AS_CLIENT(win) != client)
+                if (win && WINDOW_IS_CLIENT(win) &&
+                    WINDOW_AS_CLIENT(win) != client)
+                {
                     sibling = WINDOW_AS_CLIENT(win);
+                }
+                else
+                    /* an invalid sibling was specified so don't restack at
+                       all, it won't make sense no matter what we do */
+                    ok = FALSE;
             }
 
-            /* activate it rather than just focus it */
-            if (!config_focus_under_mouse)
-                ignore_start = event_start_ignore_all_enters();
-            stacking_restack_request(client, sibling,
-                                     e->xconfigurerequest.detail,
-                                     TRUE);
-            if (!config_focus_under_mouse)
-                event_end_ignore_all_enters(ignore_start);
+            if (ok) {
+                if (!config_focus_under_mouse)
+                    ignore_start = event_start_ignore_all_enters();
+                stacking_restack_request(client, sibling,
+                                         e->xconfigurerequest.detail);
+                if (!config_focus_under_mouse)
+                    event_end_ignore_all_enters(ignore_start);
+            }
 
-            /* if a stacking change moves the window without resizing */
+            /* a stacking change moves the window without resizing */
             move = TRUE;
         }
 
@@ -1160,7 +1161,6 @@ static void event_handle_client(ObClient *client, XEvent *e)
 
         {
             gint lw,lh;
-            gulong ignore_start;
 
             client_try_configure(client, &x, &y, &w, &h, &lw, &lh, FALSE);
 
@@ -1179,9 +1179,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
 
             ob_debug("Granting ConfigureRequest x %d y %d w %d h %d\n",
                      x, y, w, h);
-            ignore_start = event_start_ignore_all_enters();
-            client_configure(client, x, y, w, h, FALSE, TRUE);
-            event_end_ignore_all_enters(ignore_start);
+            client_configure(client, x, y, w, h, FALSE, TRUE, TRUE);
         }
         break;
     }
@@ -1214,7 +1212,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
         /* we don't want the reparent event, put it back on the stack for the
            X server to deal with after we unmanage the window */
         XPutBackEvent(ob_display, e);
-     
+
         ob_debug("ReparentNotify for window 0x%x\n", client->window);
         client_unmanage(client);
         break;
@@ -1225,7 +1223,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
                                        it can happen now when the window is on
                                        another desktop, but we still don't
                                        want it! */
-        client_activate(client, FALSE, TRUE);
+        client_activate(client, FALSE, TRUE, TRUE, TRUE);
         break;
     case ClientMessage:
         /* validate cuz we query stuff off the client here */
@@ -1264,7 +1262,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
             if ((unsigned)e->xclient.data.l[0] < screen_num_desktops ||
                 (unsigned)e->xclient.data.l[0] == DESKTOP_ALL)
                 client_set_desktop(client, (unsigned)e->xclient.data.l[0],
-                                   FALSE);
+                                   FALSE, FALSE);
         } else if (msgtype == prop_atoms.net_wm_state) {
             gulong ignore_start;
 
@@ -1303,7 +1301,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
                 ob_debug_type(OB_DEBUG_APP_BUGS,
                               "_NET_ACTIVE_WINDOW message for window %s is "
                               "missing source indication\n");
-            client_activate(client, FALSE,
+            client_activate(client, FALSE, TRUE, TRUE,
                             (e->xclient.data.l[0] == 0 ||
                              e->xclient.data.l[0] == 2));
         } else if (msgtype == prop_atoms.net_wm_moveresize) {
@@ -1343,7 +1341,6 @@ static void event_handle_client(ObClient *client, XEvent *e)
                 moveresize_end(TRUE);
         } else if (msgtype == prop_atoms.net_moveresize_window) {
             gint ograv, x, y, w, h;
-            gulong ignore_start;
 
             ograv = client->gravity;
 
@@ -1388,10 +1385,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
 
             client_find_onscreen(client, &x, &y, w, h, FALSE);
 
-            /* ignore enter events caused by these like ob actions do */
-            ignore_start = event_start_ignore_all_enters();
-            client_configure(client, x, y, w, h, FALSE, TRUE);
-            event_end_ignore_all_enters(ignore_start);
+            client_configure(client, x, y, w, h, FALSE, TRUE, FALSE);
 
             client->gravity = ograv;
         } else if (msgtype == prop_atoms.net_restack_window) {
@@ -1428,13 +1422,13 @@ static void event_handle_client(ObClient *client, XEvent *e)
                         ignore_start = event_start_ignore_all_enters();
                     /* just raise, don't activate */
                     stacking_restack_request(client, sibling,
-                                             e->xclient.data.l[2], FALSE);
+                                             e->xclient.data.l[2]);
                     if (!config_focus_under_mouse)
                         event_end_ignore_all_enters(ignore_start);
 
                     /* send a synthetic ConfigureNotify, cuz this is supposed
                        to be like a ConfigureRequest. */
-                    client_reconfigure(client);
+                    client_reconfigure(client, TRUE);
                 } else
                     ob_debug_type(OB_DEBUG_APP_BUGS,
                                   "_NET_RESTACK_WINDOW sent for window %s "
@@ -1446,7 +1440,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
     case PropertyNotify:
         /* validate cuz we query stuff off the client here */
         if (!client_validate(client)) break;
-  
+
         /* compress changes to a single property into a single change */
         while (XCheckTypedWindowEvent(ob_display, client->window,
                                       e->type, &ce)) {
@@ -1481,25 +1475,15 @@ static void event_handle_client(ObClient *client, XEvent *e)
 
         msgtype = e->xproperty.atom;
         if (msgtype == XA_WM_NORMAL_HINTS) {
-            gint x, y, w, h, lw, lh;
-
             ob_debug("Update NORMAL hints\n");
             client_update_normal_hints(client);
             /* normal hints can make a window non-resizable */
             client_setup_decor_and_functions(client, FALSE);
 
-            /* make sure the client's sizes are within its bounds */
-            RECT_TO_DIMS(client->area, x, y, w, h);
-            client_try_configure(client, &x, &y, &w, &h, &lw, &lh, FALSE);
-            if (!RECT_EQUAL_DIMS(client->area, x, y, w, h)) {
-                gulong ignore_start;
-
-                ob_debug("Configuring client x %d y %d w %d h %d\n",
-                         x, y, w, h);
-                ignore_start = event_start_ignore_all_enters();
-                client_configure(client, x, y, w, h, FALSE, TRUE);
-                event_end_ignore_all_enters(ignore_start);
-            }
+            /* make sure the client's sizes are within its bounds, but only
+               reconfigure the window if it needs to. emacs will update its
+               normal hints every time it receives a conigurenotify */
+            client_reconfigure(client, FALSE);
         } else if (msgtype == XA_WM_HINTS) {
             client_update_wmhints(client);
         } else if (msgtype == XA_WM_TRANSIENT_FOR) {
@@ -1629,7 +1613,7 @@ static gboolean event_handle_menu_keyboard(XEvent *ev)
     guint keycode, state;
     gunichar unikey;
     ObMenuFrame *frame;
-    gboolean ret = TRUE;
+    gboolean ret = FALSE;
 
     keycode = ev->xkey.keycode;
     state = ev->xkey.state;
@@ -1637,100 +1621,112 @@ static gboolean event_handle_menu_keyboard(XEvent *ev)
 
     frame = find_active_or_last_menu();
     if (frame == NULL)
-        ret = FALSE;
+        g_assert_not_reached(); /* there is no active menu */
 
-    else if (keycode == ob_keycode(OB_KEY_ESCAPE) && state == 0)
-        menu_frame_hide_all();
-
-    else if (keycode == ob_keycode(OB_KEY_RETURN) && (state == 0 ||
-                                                      state == ControlMask))
-    {
-        /* Enter runs the active item or goes into the submenu.
-           Control-Enter runs it without closing the menu. */
-        if (frame->child)
-            menu_frame_select_next(frame->child);
-        else if (frame->selected)
-            menu_entry_frame_execute(frame->selected, state, ev->xkey.time);
-    }
-
-    else if (keycode == ob_keycode(OB_KEY_LEFT) && ev->xkey.state == 0) {
-        /* Left goes to the parent menu */
-        menu_frame_select(frame, NULL, TRUE);
-    }
-
-    else if (keycode == ob_keycode(OB_KEY_RIGHT) && ev->xkey.state == 0) {
-        /* Right goes to the selected submenu */
-        if (frame->child) menu_frame_select_next(frame->child);
-    }
-
-    else if (keycode == ob_keycode(OB_KEY_UP) && state == 0) {
-        menu_frame_select_previous(frame);
-    }
-
-    else if (keycode == ob_keycode(OB_KEY_DOWN) && state == 0) {
-        menu_frame_select_next(frame);
-    }
-
-    /* keyboard accelerator shortcuts. (allow controlmask) */
-    else if ((ev->xkey.state & ~ControlMask) == 0 &&
-             /* was it a valid key? */
-             unikey != 0 &&
-             /* don't bother if the menu is empty. */
-             frame->entries)
-    {
-        GList *start;
-        GList *it;
-        ObMenuEntryFrame *found = NULL;
-        guint num_found = 0;
-
-        /* start after the selected one */
-        start = frame->entries;
-        if (frame->selected) {
-            for (it = start; frame->selected != it->data; it = g_list_next(it))
-                g_assert(it != NULL); /* nothing was selected? */
-            /* next with wraparound */
-            start = g_list_next(it);
-            if (start == NULL) start = frame->entries;
+    /* Allow control while going thru the menu */
+    else if (ev->type == KeyPress && (state & ~ControlMask) == 0) {
+        if (keycode == ob_keycode(OB_KEY_ESCAPE)) {
+            menu_frame_hide_all();
+            ret = TRUE;
         }
 
-        it = start;
-        do {
-            ObMenuEntryFrame *e = it->data;
-            gunichar entrykey = 0;
+        else if (keycode == ob_keycode(OB_KEY_LEFT)) {
+            /* Left goes to the parent menu */
+            menu_frame_select(frame, NULL, TRUE);
+            ret = TRUE;
+        }
 
-            if (e->entry->type == OB_MENU_ENTRY_TYPE_NORMAL)
-                entrykey = e->entry->data.normal.shortcut;
-            else if (e->entry->type == OB_MENU_ENTRY_TYPE_SUBMENU)
-                entrykey = e->entry->data.submenu.submenu->shortcut;
+        else if (keycode == ob_keycode(OB_KEY_RIGHT)) {
+            /* Right goes to the selected submenu */
+            if (frame->child) menu_frame_select_next(frame->child);
+            ret = TRUE;
+        }
 
-            if (unikey == entrykey) {
-                if (found == NULL) found = e;
-                ++num_found;
-            }
+        else if (keycode == ob_keycode(OB_KEY_UP)) {
+            menu_frame_select_previous(frame);
+            ret = TRUE;
+        }
 
-            /* next with wraparound */
-            it = g_list_next(it);
-            if (it == NULL) it = frame->entries;
-        } while (it != start);
-
-        if (found) {
-            if (found->entry->type == OB_MENU_ENTRY_TYPE_NORMAL &&
-                num_found == 1)
-            {
-                menu_frame_select(frame, found, TRUE);
-                usleep(50000); /* highlight the item for a short bit so the
-                                  user can see what happened */
-                menu_entry_frame_execute(found, state, ev->xkey.time);
-            } else {
-                menu_frame_select(frame, found, TRUE);
-                if (num_found == 1)
-                    menu_frame_select_next(frame->child);
+        else if (keycode == ob_keycode(OB_KEY_DOWN)) {
+            menu_frame_select_next(frame);
+            ret = TRUE;
+        }
+    }
+
+    /* Use KeyRelease events for running things so that the key release doesn't
+       get sent to the focused application.
+
+       Allow ControlMask only, and don't bother if the menu is empty */
+    else if (ev->type == KeyRelease && (state & ~ControlMask) == 0 &&
+             frame->entries)
+    {
+        if (keycode == ob_keycode(OB_KEY_RETURN)) {
+            /* Enter runs the active item or goes into the submenu.
+               Control-Enter runs it without closing the menu. */
+            if (frame->child)
+                menu_frame_select_next(frame->child);
+            else if (frame->selected)
+                menu_entry_frame_execute(frame->selected, state);
+
+            ret = TRUE;
+        }
+
+        /* keyboard accelerator shortcuts. (if it was a valid key) */
+        else if (unikey != 0) {
+            GList *start;
+            GList *it;
+            ObMenuEntryFrame *found = NULL;
+            guint num_found = 0;
+
+            /* start after the selected one */
+            start = frame->entries;
+            if (frame->selected) {
+                for (it = start; frame->selected != it->data;
+                     it = g_list_next(it))
+                    g_assert(it != NULL); /* nothing was selected? */
+                /* next with wraparound */
+                start = g_list_next(it);
+                if (start == NULL) start = frame->entries;
+            }
+
+            it = start;
+            do {
+                ObMenuEntryFrame *e = it->data;
+                gunichar entrykey = 0;
+
+                if (e->entry->type == OB_MENU_ENTRY_TYPE_NORMAL)
+                    entrykey = e->entry->data.normal.shortcut;
+                else if (e->entry->type == OB_MENU_ENTRY_TYPE_SUBMENU)
+                    entrykey = e->entry->data.submenu.submenu->shortcut;
+
+                if (unikey == entrykey) {
+                    if (found == NULL) found = e;
+                    ++num_found;
+                }
+
+                /* next with wraparound */
+                it = g_list_next(it);
+                if (it == NULL) it = frame->entries;
+            } while (it != start);
+
+            if (found) {
+                if (found->entry->type == OB_MENU_ENTRY_TYPE_NORMAL &&
+                    num_found == 1)
+                {
+                    menu_frame_select(frame, found, TRUE);
+                    usleep(50000); /* highlight the item for a short bit so the
+                                      user can see what happened */
+                    menu_entry_frame_execute(found, state);
+                } else {
+                    menu_frame_select(frame, found, TRUE);
+                    if (num_found == 1)
+                        menu_frame_select_next(frame->child);
+                }
+
+                ret = TRUE;
             }
-        } else
-            ret = FALSE;
+        }
     }
-    else
-        ret = FALSE;
 
     return ret;
 }
@@ -1750,8 +1746,7 @@ static gboolean event_handle_menu(XEvent *ev)
                                             ev->xbutton.y_root)))
             {
                 menu_frame_select(e->frame, e, TRUE);
-                menu_entry_frame_execute(e, ev->xbutton.state,
-                                         ev->xbutton.time);
+                menu_entry_frame_execute(e, ev->xbutton.state);
             }
             else
                 menu_frame_hide_all();
@@ -1780,8 +1775,8 @@ static gboolean event_handle_menu(XEvent *ev)
             menu_frame_select(e->frame, NULL, FALSE);
         }
         break;
-    case MotionNotify:   
-        if ((e = menu_entry_frame_under(ev->xmotion.x_root,   
+    case MotionNotify:
+        if ((e = menu_entry_frame_under(ev->xmotion.x_root,
                                         ev->xmotion.y_root)))
             if (!(f = find_active_menu()) ||
                 f == e->frame ||
@@ -1790,6 +1785,7 @@ static gboolean event_handle_menu(XEvent *ev)
                 menu_frame_select(e->frame, e, FALSE);
         break;
     case KeyPress:
+    case KeyRelease:
         ret = event_handle_menu_keyboard(ev);
         break;
     }
@@ -1812,9 +1808,7 @@ static void event_handle_user_input(ObClient *client, XEvent *e)
 
     /* if the keyboard interactive action uses the event then dont
        use it for bindings. likewise is moveresize uses the event. */
-    if (!keyboard_process_interactive_grab(e, &client) &&
-        !(moveresize_in_progress && moveresize_event(e)))
-    {
+    if (!actions_interactive_input_event(e) && !moveresize_event(e)) {
         if (moveresize_in_progress)
             /* make further actions work on the client being
                moved/resized */
@@ -1828,10 +1822,9 @@ static void event_handle_user_input(ObClient *client, XEvent *e)
                in the case where it is animating before disappearing */
             if (!client || !frame_iconify_animating(client->frame))
                 mouse_event(client, e);
-        } else if (e->type == KeyPress) {
+        } else
             keyboard_event((focus_cycle_target ? focus_cycle_target :
                             (client ? client : focus_client)), e);
-        }
     }
 }
 
@@ -1920,21 +1913,24 @@ static gboolean is_enter_focus_event_ignored(XEvent *e)
 
 void event_cancel_all_key_grabs()
 {
-    if (keyboard_interactively_grabbed())
-        keyboard_interactive_cancel();
-    else if (menu_frame_visible)
+    if (actions_interactive_act_running()) {
+        actions_interactive_cancel_act();
+        ob_debug("KILLED interactive action\n");
+    }
+    else if (menu_frame_visible) {
         menu_frame_hide_all();
-    else if (grab_on_keyboard())
+        ob_debug("KILLED open menus\n");
+    }
+    else if (moveresize_in_progress) {
+        moveresize_end(TRUE);
+        ob_debug("KILLED interactive moveresize\n");
+    }
+    else if (grab_on_keyboard()) {
         ungrab_keyboard();
+        ob_debug("KILLED active grab on keyboard\n");
+    }
     else
-        /* If we don't have the keyboard grabbed, then ungrab it with
-           XUngrabKeyboard, so that there is not a passive grab left
-           on from the KeyPress. If the grab is left on, and focus
-           moves during that time, it will be NotifyWhileGrabbed, and
-           applications like to ignore those! */
-        if (!keyboard_interactively_grabbed())
-            XUngrabKeyboard(ob_display, CurrentTime);
-
+        ungrab_passive_key();
 }
 
 gboolean event_time_after(Time t1, Time t2)
