@@ -23,8 +23,11 @@
 #include "frame.h"
 #include "focus.h"
 #include "config.h"
+#include "dock.h"
 #include "debug.h"
 
+extern ObDock *dock;
+
 static void add_choice(guint *choice, guint mychoice)
 {
     guint i;
@@ -43,19 +46,20 @@ static Rect *pick_pointer_head(ObClient *c)
     guint i;
     gint px, py;
 
-    screen_pointer_pos(&px, &py);
-     
-    for (i = 0; i < screen_num_monitors; ++i) {  
-        if (RECT_CONTAINS(*screen_physical_area_monitor(i), px, py)) {
-            return screen_area_monitor(c->desktop, i);
+    if (screen_pointer_pos(&px, &py)) {
+        for (i = 0; i < screen_num_monitors; ++i) {
+            Rect *monitor = screen_physical_area_monitor(i);
+            gboolean contain = RECT_CONTAINS(*monitor, px, py);
+            g_free(monitor);
+            if (contain)
+                return screen_area(c->desktop, i, NULL);
         }
-    }
-    g_assert_not_reached();
+        g_assert_not_reached();
+    } else
+        return NULL;
 }
 
-/*! Pick a monitor to place a window on.
-  The returned array value should be freed with g_free. The areas within the
-  array should not be freed. */
+/*! Pick a monitor to place a window on. */
 static Rect **pick_head(ObClient *c)
 {
     Rect **area;
@@ -82,7 +86,7 @@ static Rect **pick_head(ObClient *c)
 
         /* try on the client's desktop */
         for (it = c->group->members; it; it = g_slist_next(it)) {
-            ObClient *itc = it->data;            
+            ObClient *itc = it->data;
             if (itc != c &&
                 (itc->desktop == c->desktop ||
                  itc->desktop == DESKTOP_ALL || c->desktop == DESKTOP_ALL))
@@ -95,7 +99,7 @@ static Rect **pick_head(ObClient *c)
 
         /* try on all desktops */
         for (it = c->group->members; it; it = g_slist_next(it)) {
-            ObClient *itc = it->data;            
+            ObClient *itc = it->data;
             if (itc != c) {
                 add_choice(choice, client_monitor(it->data));
                 ob_debug("placement adding choice %d for group sibling on "
@@ -112,19 +116,23 @@ static Rect **pick_head(ObClient *c)
 
     screen_pointer_pos(&px, &py);
 
-    for (i = 0; i < screen_num_monitors; i++)
-        if (RECT_CONTAINS(*screen_physical_area_monitor(i), px, py)) {
+    for (i = 0; i < screen_num_monitors; i++) {
+        Rect *monitor = screen_physical_area_monitor(i);
+        gboolean contain = RECT_CONTAINS(*monitor, px, py);
+        g_free(monitor);
+        if (contain) {
             add_choice(choice, i);
             ob_debug("placement adding choice %d for mouse pointer\n", i);
             break;
         }
+    }
 
     /* add any leftover choices */
     for (i = 0; i < screen_num_monitors; ++i)
         add_choice(choice, i);
 
     for (i = 0; i < screen_num_monitors; ++i)
-        area[i] = screen_area_monitor(c->desktop, choice[i]);
+        area[i] = screen_area(c->desktop, choice[i], NULL);
 
     return area;
 }
@@ -148,6 +156,8 @@ static gboolean place_random(ObClient *client, gint *x, gint *y)
     if (b > t) *y = g_random_int_range(t, b + 1);
     else       *y = areas[i]->y;
 
+    for (i = 0; i < screen_num_monitors; ++i)
+        g_free(areas[i]);
     g_free(areas);
 
     return TRUE;
@@ -213,176 +223,129 @@ static GSList* area_remove(GSList *list, Rect *a)
     return result;
 }
 
-static gint area_cmp(gconstpointer p1, gconstpointer p2, gpointer data)
+enum {
+    IGNORE_FULLSCREEN = 1,
+    IGNORE_MAXIMIZED  = 2,
+    IGNORE_MENUTOOL   = 3,
+    /*IGNORE_SHADED     = 3,*/
+    IGNORE_NONGROUP   = 4,
+    IGNORE_BELOW      = 5,
+    /*IGNORE_NONFOCUS   = 1 << 5,*/
+    IGNORE_DOCK       = 6,
+    IGNORE_END        = 7
+};
+
+static gboolean place_nooverlap(ObClient *c, gint *x, gint *y)
 {
-    ObClient *c = data;
-    Rect *carea = &c->frame->area;
-    const Rect *a1 = p1, *a2 = p2;
-    gboolean diffhead = FALSE;
-    guint i;
-    Rect *a;
-
-    for (i = 0; i < screen_num_monitors; ++i) {
-        a = screen_physical_area_monitor(i);
-        if (RECT_CONTAINS(*a, a1->x, a1->y) &&
-            !RECT_CONTAINS(*a, a2->x, a2->y))
-        {
-            diffhead = TRUE;
-            break;
-        }
-    }
-
-    /* has to be more than me in the group */
-    if (diffhead && client_has_group_siblings(c)) {
-        guint *num, most;
-        GSList *it;
-
-        /* find how many clients in the group are on each monitor, use the
-           monitor with the most in it */
-        num = g_new0(guint, screen_num_monitors);
-        for (it = c->group->members; it; it = g_slist_next(it))
-            if (it->data != c)
-                ++num[client_monitor(it->data)];
-        most = 0;
-        for (i = 1; i < screen_num_monitors; ++i)
-            if (num[i] > num[most])
-                most = i;
-
-        g_free(num);
-
-        a = screen_physical_area_monitor(most);
-        if (RECT_CONTAINS(*a, a1->x, a1->y))
-            return -1;
-        if (RECT_CONTAINS(*a, a2->x, a2->y))
-            return 1;
-    }
-
-    return MIN((a1->width - carea->width), (a1->height - carea->height)) -
-        MIN((a2->width - carea->width), (a2->height - carea->height));
-}
-
-typedef enum
-{
-    SMART_FULL,
-    SMART_GROUP,
-    SMART_FOCUSED
-} ObSmartType;
-
-#define SMART_IGNORE(placer, c) \
-    (placer == c || c->shaded || !c->frame->visible || \
-     c->type == OB_CLIENT_TYPE_SPLASH || c->type == OB_CLIENT_TYPE_DESKTOP || \
-     ((c->type == OB_CLIENT_TYPE_MENU || c->type == OB_CLIENT_TYPE_TOOLBAR) &&\
-      client_has_parent(c)) || \
-     (c->desktop != DESKTOP_ALL && \
-      c->desktop != (placer->desktop == DESKTOP_ALL ? \
-                     screen_desktop : placer->desktop)))
-
-static gboolean place_smart(ObClient *client, gint *x, gint *y,
-                            ObSmartType type, gboolean ignore_max)
-{
-    gboolean ret = FALSE;
-    GSList *spaces = NULL, *sit;
-    GList *it;
     Rect **areas;
+    gint ignore;
+    gboolean ret;
+    gint maxsize;
+    GSList *spaces = NULL, *sit, *maxit;
     guint i;
 
-    if (type == SMART_GROUP) {
-        /* has to be more than me in the group */
-        if (!client_has_group_siblings(client))
-            return FALSE;
-    }
+    areas = pick_head(c);
+    ret = FALSE;
+    maxsize = 0;
+    maxit = NULL;
 
-    areas = pick_head(client);
+    /* try ignoring different things to find empty space */
+    for (ignore = 0; ignore < IGNORE_END && !ret; ignore++) {
+        guint i;
 
-    for (i = 0; i < screen_num_monitors && !ret; ++i) {
-        spaces = area_add(spaces, areas[i]);
+        /* try all monitors in order of preference */
+        for (i = 0; i < screen_num_monitors && !ret; ++i) {
+            GList *it;
 
-        /* stay out from under windows in higher layers */
-        for (it = stacking_list; it; it = g_list_next(it)) {
-            ObClient *c;
+            /* add the whole monitor */
+            spaces = area_add(spaces, areas[i]);
 
-            if (WINDOW_IS_CLIENT(it->data)) {
-                c = it->data;
-                if (ignore_max &&
-                    (c->fullscreen || (c->max_vert && c->max_horz)))
-                    continue;
-            } else
-                continue;
+            /* go thru all the windows */
+            for (it = client_list; it; it = g_list_next(it)) {
+                ObClient *test = it->data;
 
-            if (c->layer > client->layer) {
-                if (!SMART_IGNORE(client, c))
-                    spaces = area_remove(spaces, &c->frame->area);
-            } else
-                break;
-        }
-
-        if (type == SMART_FULL || type == SMART_FOCUSED) {
-            gboolean found_foc = FALSE, stop = FALSE;
-            ObClient *foc;
-
-            foc = focus_order_find_first(client->desktop == DESKTOP_ALL ?
-                                         screen_desktop : client->desktop);
-
-            for (; it && !stop; it = g_list_next(it)) {
-                ObClient *c;
-
-                if (WINDOW_IS_CLIENT(it->data)) {
-                    c = it->data;
-                    if (ignore_max &&
-                        (c->fullscreen || (c->max_vert && c->max_horz)))
-                        continue;
-                } else
-                    continue;
-
-                if (!SMART_IGNORE(client, c)) {
-                    if (type == SMART_FOCUSED)
-                        if (found_foc)
-                            stop = TRUE;
-                    if (!stop)
-                        spaces = area_remove(spaces, &c->frame->area);
+                /* should we ignore this client? */
+                if (screen_showing_desktop) continue;
+                if (c == test) continue;
+                if (test->iconic) continue;
+                if (c->desktop != DESKTOP_ALL) {
+                    if (test->desktop != c->desktop &&
+                        test->desktop != DESKTOP_ALL) continue;
+                } else {
+                    if (test->desktop != screen_desktop &&
+                        test->desktop != DESKTOP_ALL) continue;
                 }
+                if (test->type == OB_CLIENT_TYPE_SPLASH ||
+                    test->type == OB_CLIENT_TYPE_DESKTOP) continue;
 
-                if (c == foc)
-                    found_foc = TRUE;
+
+                if ((ignore >= IGNORE_FULLSCREEN) &&
+                    test->fullscreen) continue;
+                if ((ignore >= IGNORE_MAXIMIZED) &&
+                    test->max_horz && test->max_vert) continue;
+                if ((ignore >= IGNORE_MENUTOOL) &&
+                    (test->type == OB_CLIENT_TYPE_MENU ||
+                     test->type == OB_CLIENT_TYPE_TOOLBAR) &&
+                    client_has_parent(c)) continue;
+                /*
+                if ((ignore >= IGNORE_SHADED) &&
+                    test->shaded) continue;
+                */
+                if ((ignore >= IGNORE_NONGROUP) &&
+                    client_has_group_siblings(c) &&
+                    test->group != c->group) continue;
+                if ((ignore >= IGNORE_BELOW) &&
+                    test->layer < c->layer) continue;
+                /*
+                if ((ignore >= IGNORE_NONFOCUS) &&
+                    focus_client != test) continue;
+                */
+                /* don't ignore this window, so remove it from the available
+                   area */
+                spaces = area_remove(spaces, &test->frame->area);
             }
-        } else if (type == SMART_GROUP) {
-            for (sit = client->group->members; sit; sit = g_slist_next(sit)) {
-                ObClient *c = sit->data;
-                if (!SMART_IGNORE(client, c))
-                    spaces = area_remove(spaces, &c->frame->area);
+
+            if (ignore < IGNORE_DOCK) {
+                Rect a;
+                dock_get_area(&a);
+                spaces = area_remove(spaces, &a);
             }
-        } else
-            g_assert_not_reached();
 
-        spaces = g_slist_sort_with_data(spaces, area_cmp, client);
+            for (sit = spaces; sit; sit = g_slist_next(sit)) {
+                Rect *r = sit->data;
 
-        for (sit = spaces; sit; sit = g_slist_next(sit)) {
-            Rect *r = sit->data;
-
-            if (!ret) {
-                if (r->width >= client->frame->area.width &&
-                    r->height >= client->frame->area.height) {
-                    ret = TRUE;
-                    if (client->type == OB_CLIENT_TYPE_DIALOG ||
-                        type != SMART_FULL)
-                    {
-                        *x = r->x + (r->width - client->frame->area.width)/2;
-                        *y = r->y + (r->height - client->frame->area.height)/2;
-                    } else {
-                        *x = r->x;
-                        *y = r->y;
-                    }
+                if (r->width >= c->frame->area.width &&
+                    r->height >= c->frame->area.height &&
+                    r->width > maxsize)
+                {
+                    maxsize = r->width;
+                    maxit = sit;
                 }
             }
 
-            g_free(r);
+            if (maxit) {
+                Rect *r = maxit->data;
+
+                /* center it in the area */
+                *x = r->x;
+                *y = r->y;
+                if (config_place_center) {
+                    *x += (r->width - c->frame->area.width) / 2;
+                    *y += (r->height - c->frame->area.height) / 2;
+                }
+                ret = TRUE;
+            }
+
+            while (spaces) {
+                g_free(spaces->data);
+                spaces = g_slist_delete_link(spaces, spaces);
+            }
         }
-        g_slist_free(spaces);
-        spaces = NULL;
     }
 
+    for (i = 0; i < screen_num_monitors; ++i)
+        g_free(areas[i]);
     g_free(areas);
-
     return ret;
 }
 
@@ -392,8 +355,9 @@ static gboolean place_under_mouse(ObClient *client, gint *x, gint *y)
     gint px, py;
     Rect *area;
 
+    if (!screen_pointer_pos(&px, &py))
+        return FALSE;
     area = pick_pointer_head(client);
-    screen_pointer_pos(&px, &py);
 
     l = area->x;
     t = area->y;
@@ -418,16 +382,25 @@ static gboolean place_per_app_setting(ObClient *client, gint *x, gint *y,
 
     /* Find which head the pointer is on */
     if (settings->monitor == 0)
+        /* this can return NULL */
         screen = pick_pointer_head(client);
     else if (settings->monitor > 0 &&
              (guint)settings->monitor <= screen_num_monitors)
-        screen = screen_area_monitor(client->desktop,
-                                     (guint)settings->monitor - 1);
-    else {
-        Rect **all = NULL;
-        all = pick_head(client);
-        screen = all[0];
-        g_free(all); /* the areas themselves don't need to be freed */
+        screen = screen_area(client->desktop, (guint)settings->monitor - 1,
+                             NULL);
+
+    /* if we have't found a screen yet.. */
+    if (!screen) {
+        Rect **areas;
+        guint i;
+
+        areas = pick_head(client);
+        screen = areas[0];
+
+        /* don't free the first one, it's being set as "screen" */
+        for (i = 1; i < screen_num_monitors; ++i)
+            g_free(areas[i]);
+        g_free(areas);
     }
 
     if (settings->center_x)
@@ -446,6 +419,7 @@ static gboolean place_per_app_setting(ObClient *client, gint *x, gint *y,
     else
         *y = screen->y + settings->position.y;
 
+    g_free(screen);
     return TRUE;
 }
 
@@ -472,7 +446,7 @@ static gboolean place_transient_splash(ObClient *client, gint *x, gint *y)
                 }
             }
             if (!first) {
-                *x = ((r + 1 - l) - client->frame->area.width) / 2 + l; 
+                *x = ((r + 1 - l) - client->frame->area.width) / 2 + l;
                 *y = ((b + 1 - t) - client->frame->area.height) / 2 + t;
                 return TRUE;
             }
@@ -483,12 +457,15 @@ static gboolean place_transient_splash(ObClient *client, gint *x, gint *y)
         client->type == OB_CLIENT_TYPE_SPLASH)
     {
         Rect **areas;
+        guint i;
 
         areas = pick_head(client);
 
         *x = (areas[0]->width - client->frame->area.width) / 2 + areas[0]->x;
         *y = (areas[0]->height - client->frame->area.height) / 2 + areas[0]->y;
 
+        for (i = 0; i < screen_num_monitors; ++i)
+            g_free(areas[i]);
         g_free(areas);
         return TRUE;
     }
@@ -500,24 +477,26 @@ static gboolean place_transient_splash(ObClient *client, gint *x, gint *y)
 gboolean place_client(ObClient *client, gint *x, gint *y,
                       ObAppSettings *settings)
 {
-    gboolean ret = FALSE;
-    if (client->positioned)
+    gboolean ret;
+    gboolean userplaced = FALSE;
+
+    /* per-app settings override program specified position
+     * but not user specified */
+    if ((client->positioned & USPosition) ||
+        ((client->positioned & PPosition) &&
+         !(settings && settings->pos_given)))
         return FALSE;
-    if (place_transient_splash(client, x, y))
-        ret = TRUE;
-    else if (!(
-        place_per_app_setting(client, x, y, settings) ||
-        ((config_place_policy == OB_PLACE_POLICY_MOUSE) ?
-         place_under_mouse(client, x, y) :
-         place_smart(client, x, y, SMART_FULL, FALSE)   ||
-         place_smart(client, x, y, SMART_FULL, TRUE)    ||
-         place_smart(client, x, y, SMART_GROUP, FALSE)  ||
-         place_smart(client, x, y, SMART_GROUP, TRUE)   ||
-         place_smart(client, x, y, SMART_FOCUSED, TRUE) ||
-         place_random(client, x, y))))
-        g_assert_not_reached(); /* the last one better succeed */
+
+    /* try a number of methods */
+    ret = place_transient_splash(client, x, y) ||
+        (userplaced = place_per_app_setting(client, x, y, settings)) ||
+        (config_place_policy == OB_PLACE_POLICY_MOUSE &&
+         place_under_mouse(client, x, y)) ||
+        place_nooverlap(client, x, y) ||
+        place_random(client, x, y);
+    g_assert(ret);
+
     /* get where the client should be */
-    frame_frame_gravity(client->frame, x, y,
-                        client->area.width, client->area.height);
-    return ret;
+    frame_frame_gravity(client->frame, x, y);
+    return !userplaced;
 }
